
const GMenu = imports.gi.GMenu;
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Gettext = imports.gettext.domain('gnome-shell-extensions-settings-menu');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const SETTINGS_HIERARCHY_KEY = 'hierarchy';

let hierarchy_signal_id;
let indicator;
let settings;
let settings_item;

const SettingsMenu = new Lang.Class({
    Name: 'SettingsMenu.SettingsMenu',
    Extends: PanelMenu.SystemStatusButton,

    _init: function() {
        this.parent('preferences-system-symbolic');

        this._app_sys = Shell.AppSystem.get_default();

        let user_menu = Main.panel.statusArea.userMenu;

        let cc_text = this._app_sys.lookup_setting('gnome-control-center.desktop').get_name();
        let user_items = user_menu.menu._getMenuItems();

        for (key in user_items) {
            let item = user_items[key];
            if (item.label && item.label.get_children()[0].get_text() == cc_text) {
                settings_item = item;
                settings_item.actor.visible = false;
            }
        }

        this.recreate();

        this._installed_changed_id = this._app_sys.connect('installed-changed', Lang.bind(this, this.recreate));
    },

    _destroy : function() {
        this.parent();
        
        this._app_sys.disconnect(this._installed_changed_id);
    },

    basename: function(filename) {
        return new String(filename).substring(filename.lastIndexOf('/') + 1);
    },

    buildMenuHierarchy : function() {
        let tree = this._app_sys.get_settings_tree();
        let root = tree.get_root_directory();

        let iter = root.iter();
        let nextType;

        let other = [];

        while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
            if (nextType == GMenu.TreeItemType.DIRECTORY) {
                let dir = iter.get_directory();
                let item = new PopupMenu.PopupSubMenuMenuItem(dir.get_name());

                let items = this.recurseSettings(dir);

                for (key in items) {
                    item.menu.addSettingsAction(items[key].name, items[key].filename);
                }

                this.menu.addMenuItem(item);
            }
            else if (nextType == GMenu.TreeItemType.ENTRY) {
                other.push(iter.get_entry());
            }
        }

        if (other.length > 0) {
            let item = new PopupMenu.PopupSubMenuMenuItem(_("Other"));

            for (key in other) {
                let info = other[key].get_app_info();
                if (!info.get_nodisplay()) {
                    let filename = this.basename(info.get_filename());
                    item.menu.addSettingsAction(info.get_name(), filename);
                }
            }

            this.menu.addMenuItem(item);
        }
    },

    buildMenuFlat : function() {
        let tree = this._app_sys.get_settings_tree();
        let root = tree.get_root_directory();

        let items = this.recurseSettings(root);
        items.sort(function(a, b) {
            return a.name.localeCompare(b.name);
        });

        for (key in items) {
            this.menu.addSettingsAction(items[key].name, items[key].filename);
        }
    },

    recreate : function() {
        this.menu.removeAll();

        if (settings.get_boolean(SETTINGS_HIERARCHY_KEY))
            this.buildMenuHierarchy();
        else
            this.buildMenuFlat();
    },

    recurseSettings : function(dir) {
        let items = [];

        let iter = dir.iter();
        let nextType;

        while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
            if (nextType == GMenu.TreeItemType.ENTRY) {
                let entry = iter.get_entry();
                let info = entry.get_app_info();
                if (!info.get_nodisplay()) {
                    let name = info.get_name();
                    let filename = this.basename(info.get_filename());

                    item = new Object();
                    item.name = name;
                    item.filename = filename;

                    items.push(item);
                }
            } else if (nextType == GMenu.TreeItemType.DIRECTORY) {
                concat = this.recurseSettings(iter.get_directory());

                for (key in concat) {
                    items.push(concat[key]);
                }
            }
        }

        return items;
    }
});

function init() {
    Convenience.initTranslations();
    settings = Convenience.getSettings();
}

function enable() {
    indicator = new SettingsMenu();
    hierarchy_signal_id = settings.connect('changed::' + SETTINGS_HIERARCHY_KEY, Lang.bind(indicator, indicator.recreate));

    Main.panel.addToStatusArea('settings-menu', indicator);
}

function disable() {
    if (settings_item)
        settings_item.actor.visible = true;

    settings.disconnect(hierarchy_signal_id);

    indicator.destroy();
}

